"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HexGrid = exports.SquareGrid = exports.Grid = exports.Cell = void 0;
const geometry_1 = require("./geometry");
class Cell {
    ctx;
    shape;
    constructor(ctx, shape) {
        this.ctx = ctx;
        this.shape = shape;
    }
    draw(callback) {
        // Create a clipping path so that nothing can be drawn outside of the cell
        this.ctx.save();
        this.ctx.beginPath();
        this.shape.draw(this.ctx);
        this.ctx.clip();
        callback(this.ctx);
        // Disable the clipping path and restore the style
        this.ctx.restore();
    }
    fill(color) {
        this.ctx.save();
        this.ctx.beginPath();
        this.shape.draw(this.ctx);
        this.ctx.fillStyle = color;
        this.ctx.fill();
        this.ctx.restore();
    }
    absoluteOffset(offset) {
        return { left: this.shape.offset.left + offset.left, top: this.shape.offset.top + offset.top };
    }
}
exports.Cell = Cell;
class Grid {
    ctx;
    columns;
    rows;
    cells;
    constructor(ctx, columns, rows, cells) {
        this.ctx = ctx;
        this.columns = columns;
        this.rows = rows;
        this.cells = cells;
    }
}
exports.Grid = Grid;
class SquareGrid extends Grid {
    constructor(ctx, columns, rows) {
        const width = ctx.canvas.width / columns;
        const height = ctx.canvas.height / rows;
        const cells = [];
        for (let y = 0; y < rows; y++) {
            for (let x = 0; x < columns; x++)
                cells.push(new Cell(ctx, new geometry_1.Rectangle({ top: y * height, left: x * width }, { width, height })));
        }
        super(ctx, columns, rows, cells);
    }
    getCell(column, row) {
        return this.cells[row * this.columns + column];
    }
}
exports.SquareGrid = SquareGrid;
class HexGrid extends Grid {
    constructor(ctx, columns, rows, radius) {
        const height = Math.sqrt(3) * radius;
        const cells = [];
        const horizontalSpacing = 3 * radius / 2;
        const verticalSpacing = height;
        let offset = false;
        let left = -horizontalSpacing;
        let top = -verticalSpacing;
        for (let column = 0; column < columns; column++) {
            top = offset ? -verticalSpacing / 2 : -verticalSpacing;
            for (let row = 0; row < rows; row++) {
                cells.push(new Cell(ctx, new geometry_1.Hexagon({ top, left }, radius)));
                top += verticalSpacing;
            }
            left += horizontalSpacing;
            offset = !offset;
        }
        super(ctx, columns, rows, cells);
    }
}
exports.HexGrid = HexGrid;
//# sourceMappingURL=grid.js.map